% This MATLAB file try to verify the exact solution from obtained
% approximate solution for Proudman-Johnson eq.:
%  --------------------------------------------------------------------------  
%      u^(4)=f(u)=:R(u*u^(3)-u^(1)*u^(2))+sin(k*x) in (0,Pi)
%      u=u^(2)=0 on boundary of (0,Pi)
%  --------------------------------------------------------------------------  
% by Fourier-Galarkin method with interval arithmetic.
  tic
  startintlab
  intvalinit('displayinfsup');
  intvalinit('sharpivmult')  % use sharp interval matrix multiplication
  format long;
  input_data
  formatSpec = 'R = %-6d N = %-6d k = %-2d\n';
  fprintf(formatSpec,R,N,k)

  one = intval('1');  two = intval('2');  unit = intval('[-1,1]');
  Pi = intval('pi');

  fid = fopen('appb.data');
  u = fread(fid,'double');
  status = fclose(fid);
  u = intval(u);

  D = Pi/two*intval(diag(1:N))^4;  % diagonal matrix D
  G = full(D);                     % Jacobian matrix G
  for j = 1:N
    df = function_df_intval(N,R,j,u);
    G(:,j) = G(:,j) - Pi/two*df(1:N);
  end 

  [z3,z9,C,C0,C1] = norms(N,R,k,u);

  MAXIT = 10;  d0 = 1d-1;   
  w = unit*eps*ones(N,1);  alpha = eps;     % initial candidate set           
  fprintf('%5s %6s %8s %6s %8s %8s %8s %8s %10s %8s\r',...
	  'its','|z1|','|z2|','z4','z5','z6','z7','z8','|Wn|','|W*|');
  formatSpec = '%5d %7.2e %7.2e %7.2e %7.2e %7.2e %7.2e %7.2e %7.2e %7.2e\n';
  for j = 1:MAXIT
    w = (1+d0)*w;  alpha = (1+d0)*alpha;    % inflation of candidate set
    [z1,z2,z4,z5,z6,z7,z8] = zs(N,u,w);
    d = R*(unit*z1*C^3*alpha + z2 + 2*unit*C^3*alpha^2) + z3;
    d = G\d;                                % next Wn
    beta = sup(C*( R*( z4+alpha*(z5+z6*C) + ...
                alpha*C^2*(z8*C+z7) + ...
                alpha^2*(C0+C1*C)) + z9));  % next W*
    fprintf(formatSpec,j,max(mag(z1)),max(mag(z2)),sup(z4),sup(z5),sup(z6),...
	    sup(z7),sup(z8),max(mag(d)),beta)
    if beta > 1
      error('Verification fails!')
    end 
    res = sum(in(d,w));                     % checking contraction: finite
    if beta <= alpha                        % infinite part 
      res = res + 1;
    end 
    if res == N+1                           % verification check
      disp('Verification has been completed!'); 
      fprintf('max(Wn): %d\n',max(mag(d))); % maximum-norm of Wn
      fprintf('  alpha: %d\n',beta);        % X-norm of W*
      save('u.dat','u')                     % save approximate solution
      save('w.dat','d')                     % save Wn
      save('alpha.dat','beta')              % save W*
      break;
    end
    w = d;  alpha = beta;                   % preparing next step
  end
